@@ -192,7 +192,7 @@ path_list_async_new (const char       *uri,
 
 	pli = path_list_data_new ();
 
-	escaped = gnome_vfs_escape_host_and_path_string (uri);
+	escaped = escape_uri (uri);
 	if (escaped == NULL) {
 		path_list_data_free (pli);
 		if (f != NULL)
@@ -591,46 +591,36 @@ rmdir_recursive (const gchar *directory)
 /* -- */
 
 
-int
+gboolean
 check_permissions (const char *path,
 		   int         mode) 
 {
 	GnomeVFSFileInfo *info;
-	GnomeVFSResult    result;
+	GnomeVFSResult    vfs_result;
 	char             *escaped;
-	int	          perm;
-	
+
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_host_and_path_string (path);
-	result = gnome_vfs_get_file_info (escaped, 
-					  info, 
-					  (GNOME_VFS_FILE_INFO_DEFAULT 
-					   | GNOME_VFS_FILE_INFO_FOLLOW_LINKS));
-	
-	if (result != GNOME_VFS_OK) 
+	escaped = escape_uri (path);
+	vfs_result = gnome_vfs_get_file_info (escaped, 
+					      info, 
+					      (GNOME_VFS_FILE_INFO_DEFAULT 
+					       | GNOME_VFS_FILE_INFO_FOLLOW_LINKS
+					       | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS));
+	g_free (escaped);
+
+	if (vfs_result != GNOME_VFS_OK) 
 		return FALSE; 
 
-	perm = 7;
-	
-	if ((mode & R_OK) && (info->permissions & 
-			      (GNOME_VFS_PERM_USER_READ | 
-			       GNOME_VFS_PERM_GROUP_READ|
-			       GNOME_VFS_PERM_OTHER_READ))) 
-		perm &= ~R_OK;
+	if ((mode & R_OK) && ! (info->permissions & GNOME_VFS_PERM_ACCESS_READABLE)) 
+		return FALSE; 
 
-	if ((mode & W_OK) && (info->permissions & 
-			      (GNOME_VFS_PERM_USER_WRITE | 
-			       GNOME_VFS_PERM_GROUP_WRITE|
-			       GNOME_VFS_PERM_OTHER_WRITE)))  
-		perm &= ~W_OK;
-	
-	if ((mode & X_OK) && (info->permissions & 
-			      (GNOME_VFS_PERM_USER_EXEC | 
-			       GNOME_VFS_PERM_GROUP_EXEC|
-			       GNOME_VFS_PERM_OTHER_EXEC)))  
-		perm &= ~X_OK;
+	if ((mode & W_OK) && ! (info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE)) 
+		return FALSE; 
 
-	return (perm);
+	if ((mode & X_OK) && ! (info->permissions & GNOME_VFS_PERM_ACCESS_WRITABLE)) 
+		return FALSE; 
+
+	return TRUE;
 }
 
 
@@ -642,10 +632,11 @@ path_is_file (const char *path)
 	gboolean          is_file;
 	char             *escaped;
 
-	if (! path || ! *path) return FALSE; 
+	if (! path || ! *path) 
+		return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_host_and_path_string (path);
+	escaped = escape_uri (path);
 
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
@@ -662,6 +653,40 @@ path_is_file (const char *path)
 }
 
 
+char *
+escape_uri (const char *uri)
+{
+	const char *start = NULL;
+	const char *uri_no_method;
+	char       *method;
+	char       *epath, *euri;
+
+	if (uri == NULL)
+		return NULL;
+
+	start = strstr (uri, "://");
+	if (start != NULL) {
+		uri_no_method = start + strlen ("://");
+		method = g_strndup (uri, start - uri);
+	} else {
+		uri_no_method = uri;
+		method = NULL;
+	}
+	
+	epath = gnome_vfs_escape_host_and_path_string (uri_no_method);
+	
+	if (method != NULL) {
+		euri = g_strdup_printf ("%s://%s", method, epath);
+		g_free (epath);
+	} else
+		euri = epath;
+
+	g_free (method);
+
+	return euri;
+}
+
+
 gboolean
 path_is_dir (const char *path)
 {
@@ -674,7 +699,7 @@ path_is_dir (const char *path)
 		return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_host_and_path_string (path);
+	escaped = escape_uri (path);
 
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
@@ -726,7 +751,7 @@ get_file_size (const char *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_host_and_path_string (path);
+	escaped = escape_uri (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info,
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -753,7 +778,7 @@ get_file_mtime (const char *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_host_and_path_string (path);
+	escaped = escape_uri (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -781,7 +806,7 @@ get_file_ctime (const gchar *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_host_and_path_string (path);
+	escaped = escape_uri (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -809,7 +834,7 @@ set_file_mtime (const gchar *path,
 	file_info->mtime = mtime;
 	file_info->atime = mtime;
 
-	escaped_path = gnome_vfs_escape_host_and_path_string (path);
+	escaped_path = escape_uri (path);
 	gnome_vfs_set_file_info (escaped_path,
 				 file_info,
 				 GNOME_VFS_SET_FILE_INFO_TIME);
@@ -987,17 +1012,21 @@ gboolean
 file_is_image (const gchar *name,
 	       gboolean     fast_file_type)
 {
-	const char *result;
+	const char *result = NULL;
 	gboolean    is_an_image;
 
 	if (fast_file_type) {
 		char *n1 = g_filename_to_utf8 (name, -1, 0, 0, 0);
-		char *n2 = g_utf8_strdown (n1, -1);
-		char *n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
-		result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
-		g_free (n3);
-		g_free (n2);
-		g_free (n1);
+		if (n1 != NULL) {
+			char *n2, *n3;
+			n2 = g_utf8_strdown (n1, -1);
+			n3 = g_filename_from_utf8 (n2, -1, 0, 0, 0);
+			if (n3 != NULL)
+				result = gnome_vfs_mime_type_from_name_or_default (n3, NULL);
+			g_free (n3);
+			g_free (n2);
+			g_free (n1);
+		}
 	} else 
 		result = gnome_vfs_get_file_mime_type (name, NULL, FALSE);
 
@@ -1182,7 +1211,8 @@ ensure_dir_exists (const char *a_path,
 	char *path;
 	char *p;
 
-	if (! a_path) return FALSE;
+	if (! a_path) 
+		return FALSE;
 
 	if (path_is_dir (a_path))
 		return TRUE;
@@ -1551,7 +1581,7 @@ new_uri_from_path (const char *path)
 	char        *uri_txt;
 	GnomeVFSURI *uri;
 
-	escaped = gnome_vfs_escape_host_and_path_string (path);
+	escaped = escape_uri (path);
 	if (escaped[0] == '/')
 		uri_txt = g_strconcat ("file://", escaped, NULL);
 	else
@@ -1700,3 +1730,4 @@ is_mime_type_writable (const char *mime_type)
 	
 	return FALSE;
 }
+
